function    script_workflow_PcTK_NoCorrelation

%==========================================================================
%
%   funciton script_workflow_PcTK_NoCorrelation
%
%   This is a sample program to run cross-talk photon counting detector
%   (PCD) model program (Ref. 1). 
%
%   7/27/2017   Ver 0.9     Katsuyuki "Ken" Taguchi, Ph.D.
%                           script_workflow_PcTK
%   2/9/2018    Ver 1.0     Katsuyuki "Ken" Taguchi, Ph.D.
%                           script_workflow_PcTK_NoCorrelation
%   3/14/2018   Ver 1.02    Katsuyuki "Ken" Taguchi, Ph.D.
%                           Ref 3 info added.
%                           Note 1 in the next section on N0 corrected. 
%                           We thank Lisha Yao of Southern Medical Univ
%                           (China) for drawing our attention to it.
%                           Official site address, "About us" added
%   4/3/2018    Ver 1.1     Katsuyuki "Ken" Taguchi, Ph.D.
%                           Correction on how to use function 'normrnd'. 
%                           'sigma' for normrnd(mu, sigma) is the standard
%                           deviation, not variance. Line 263.
%               
% (1) Read clinical volume data f
% (2) Segment images to fw (water) and fb (bone)
% (3) Gen basis-sino, vw (water) and vb (bone)
% (4) Gen spectrum-sino, Sw (water) and Sb (bone)
% (5) Gen count-sino, njl (pixel, bin) for various PCDs
% (6) Recon fl from njl using FBP 
%
%   Note 1. Users can use this software for free of charge within the scope
%           of the work that has been approved by the committee. 
%   Note 2. This script generates PCD counts with correlated noise
%   Note 3. Any projector can be used for (3). We like ASTRA Toolbox these
%           days (https://www.astra-toolbox.com).
%   
%   Ref 1. 
%       K. Taguchi, K. Stierstorfer, C. Polster, O. Lee, and S. Kappler 
%       "Spatio-energetic cross-talks in photon counting detectors: 
%       Numerical detector model (PcTK) and CT workflow for image quality assessment" 
%       Medical Physics, Vol. 45(),  (2018); doi: 
%
%   Ref 2. 
%       K. Taguchi, C. Polster, O. Lee, K. Stierstorfer, and S. Kappler 
%       "Spatio-energetic cross-talks in photon counting detectors: 
%       Detector model and correlated Poisson data generator" 
%       Medical Physics, Vol. 43(12), 6388 (2016); doi: 10.1118/1.4966699
%
%   Ref 3. 
%       J. Cammin, J. Xu, W. C. Barber, J. S. Iwanczyk, N. E. Hartsough, and K. Taguchi 
%       "A cascaded model of spectral distortion due to spectral response effect
%       and pulse pileup effects in a photon-counting x-ray detector for CT" 
%       Medical Physics, Vol. 41(4), 041905 (2014); doi: 10.1118/1.4866890
%
%   The official site of PcTK is pctk.jhu.edu
%
%   We are CT research group at Division of Medical Imaging Physics, 
%   Department of Radiology, Johns Hopkins University. PcTK has been 
%   developed in collaboration with Siemens Healthineers (Forchheim,
%   Germany). We wish to help the community by making PcTK available to
%   academic researchers. Please respect our goodwill and community spirit, 
%   and follow the guidelines and rules in place. For example, 
%       - Each user should submit the signed software license agreement. 
%       - No commercial use, either directly or indirectly, is permitted. 
%
%==========================================================================

%% All of steps
%
%   Step 1. Read parameters, set parameters
%   Step 2. Read data
%           2a. Initial x-ray spectrum, PMF(E1)
%           2b. Initial number of photons per PCD pixel at each proj, N0(view)
%           2c. A set of 3-D basis sinogram, vk(ch,row,view),k=1...
%           2d. A linear attenuation coeff of material k, muk(E)
%           2e. Normalized cov matrix, nCov3x3w(9*Nl, 9*Nl, nE1), for a given energy threshold setting
%           2f. PCD parameters 
%   Step 3. Calculate a 4-D spectrum sinogram from basis sinograms of multiple materials, St(E1,ch,row,view)
%   Step 4. Calculate a noise-free 4-D count sinogram, nR(Nl,ch,row,view)
%   Step 5. Generate a noisy 4-D count sinogram, nRm(Nl,ch,row,view)
%
%   Note 1: One tube gives ~2.2*10^6 counts/(s*mA*mm^2) at 1 m away with 120 kVp (Ref. 3)
%           N0 = 2.2*10^6*tc/(nprojpersec)*(dpix/1000)^2.*(1000/Rd)^2
%              = 14,322.9 photons/projection/pixel through the air
%               tc          : tube current (mA), 600
%               nprojpersec : # of projections (/s), 4000
%               dpix        : PCD pixel size (um), 225
%               Rd          : Focus-to-PCD distance (mm), 1080
%   
%   Note 2: The effect of bowtie filter must be taken into account using m2_sino_v
%   Note 3: The heel effect on neither N0 nor spectrum is taken into account. 
%   Note 4: The sign for directory/folder is '\' for Windows, '/' for Mac and Linux.
%           Use the appropriate one. This script is designed for Mac.

%% Step 1. Read parameters, set parameters 
%   j=1...Nj: Sinogram pixel index. Nj = Nch * Nrow * Nview
%   k=1...Nk: Basis material index
%   l=1...Nl: Energy window (threshold) index
%   m=1...Nm: Energy index. m indicates m keV
%   
    addpath('./3_src');
    model_version = 3.2;
    Emax = 140;             % integer, max of possible input energy (keV)
    iflag_3x3=0;
    flag_noise_normal  = 1;
    flag_noise_Poisson = 2;
    iflag_noise = flag_noise_normal;    % flag to choose noise type
    
    fprintf('\n\n *** PcTK version %.2f. CT workflow using PcTK: \n', model_version);
    fprintf('          Generating noisy PCD data WITHOUT correlation *** \n\n');
    fprintf(' *** Warning ***\n');
    fprintf('   The PCD cross-talks are NOT fully modeled.\n');
    fprintf('   Use script_workflow_PcTK,\n');
    fprintf('   if you wish to generate correlated noise.\n\n');
    
    % Directory/folder names for input/output data and parameter files
    dirname_inputdata  = './1_inputdata/' ;
    dirname_outputdata = './2_outputdata/';
    if ~exist(dirname_outputdata,'dir')
        mkdir(dirname_outputdata);
    end

    % File names for ...
    %   2a. Initial x-ray spectrum, PMF(E1)
    filename_Stube    = [dirname_inputdata,'pmf_S0_Al2.0_120kVp.mat'];  % file for x-ray spectrum (PDF)
    
    %   2b. Initial number of photons per PCD pixel at each proj, N0(view)
    filename_N0view   = [dirname_inputdata,'v_N0.mat'];       % file for N0(view) per PCD pixel
    
    %   2c. A set of 4-D basis sinogram, vk(ch,row,view),k=1...
    filename_sino_vk  = [dirname_inputdata,'m4_sino_v.mat'];  % file for basis sinogram data k
    
    %   2d. A linear attenuation coeff of material k, muk(E)
    filename_muk      = [dirname_inputdata,'m2_mukE.csv'];    % file for mu(E) of basis materials k
    
    %   2e. PCD parameters 
    filename_PCDparam = [dirname_inputdata,'SRF_param_v32_20170622.csv'];  % file for PCD parameters
    
    %   2f. Normalized cov matrix, nCov3x3w(9*Nl, 9*Nl, nE1), for a given energy threshold setting
    %       will be created based on PCD parameters below.

    %   2g. A set of 4-D PCD counts sinogram, pcd(nb,ch,row,view),nb=1...Nl
    %       and a set of 2-D PCD counts sinogram, pcd(ch,view) for nb,row
    filename_sino_pcd_mean  = [dirname_outputdata,'m4_sino_pcd_mean' ];  % file for pcd sinogram data 
    filename_sino_pcd_noisy = [dirname_outputdata,'m4_sino_pcd_noisy'];  % file for pcd sinogram data 
%     filename_sino_g_row_win_mean  = [dirname_outputdata,'m2_sino_g_row_win_mean' ];  % file for pcd sinogram data 
%     filename_sino_g_row_win_noisy = [dirname_outputdata,'m2_sino_g_row_win_noisy'];  % file for pcd sinogram data 
    
    % Read PCD parameters
    [r0,e_sig,dpix,dz,rho_PCD,Nl,v_Eth] = func_read_params(filename_PCDparam, model_version); 
    
%%   Step 2. Read data
    
    % 2a. Initial x-ray spectrum, PMF(E1)
    load( filename_Stube );
    v_S0E = m2_S0(1:Emax,2); clear m2_S0; 
    
    % 2b. Initial number of photons per PCD pixel at each proj, N0(view)
    load( filename_N0view );
    
    % 2c. A set of 3-D basis sinogram, vk(ch,row,view),k=1...
    load( filename_sino_vk );
    Nk    = Nbasis;
        % 'm4_sino_v','Nbasis','Nch','Nrow','Nview','str_basis_type';
        % m4_sino_v(Nbasis, Nch, Nrow, Nview)
    
    % 2d. A linear attenuation coeff of material k, muk(E)
    m2_mukE = csvread( filename_muk, 1,0 );
    m2_mukE = m2_mukE(1:Emax,2:end);
    
    % 2e. Normalized cov matrix, nCov3x3w(9*Nl, 9*Nl, nE1), for a given energy threshold setting
    % Creating filename for nCov3x3w data based on PCD parameters
    filnam_nCov3x3w = ...
        sprintf('dat_nCovw_ver%.1f_dpix_%d_dz_%d_r0_%d_esig_%.1f_Eth',...
        model_version, dpix, dz, r0, e_sig);
    for iEth=1:Nl
        filnam_nCov3x3w = sprintf('%s_%.1f', filnam_nCov3x3w, v_Eth(iEth));
    end
    filnam_nCov3x3w = sprintf('%s_keV.mat',filnam_nCov3x3w);
    filename_nCov3x3w = [dirname_inputdata,filnam_nCov3x3w];  % for nCov3x3w file
    
    % Produce filename for nCov3x3E data, just in case
    filnam_nCov3x3E = ...
        sprintf('dat_nCovE_ver%.1f_dpix_%d_dz_%d_r0_%d_esig_%.1f.mat',...
        model_version, dpix, dz, r0, e_sig);
    filename_nCov3x3E = [dirname_inputdata,filnam_nCov3x3E];  % for nCov3x3E file
    
    % read nCov3x3w data generated by PcTK
    if exist(filename_nCov3x3w,'file')
        load(filename_nCov3x3w);
        % 'v_Eo','v_E1','m3_nCovBin_3x3_0','v_Eth','v_Pr_pq','-v7.3');
    else
        fprintf('File nCov3x3w does not exist. \n');
        if exist(filename_nCov3x3E,'file')
            fprintf('Attempting to create nCov3x3w from nCov3x3E.\n');
            fprintf('FYI, filename of nCov3x3w is %s\n',filename_nCov3x3w);
            load(filename_nCov3x3E);
                % 'v_Eo','v_E1','m3_nCovE_3x3_0','m2_SRE_q0','m2_SRE_q1','m2_SRE_q2','v_Pr_pq','m2_rho_e','m2_nvol');
            % calc nCov3x3w data from nCovE data
            m3_nCov3x3w=zeros(length(v_Eth)*9,length(v_Eth)*9,size(m3_nCov3x3E,3));
            for j=1:size(m3_nCov3x3w,3)
                [ m3_nCov3x3w(:,:,j) ] = func_covE2covBin( squeeze(m3_nCov3x3E(:,:,j)), v_Eo, v_Eth );
            end
            save(filename_nCov3x3w,'v_Eo','v_E1','m3_nCov3x3w','v_Eth','v_Pr_pq','-v7.3');
        else
            fprintf('File nCov3x3E does not exist as well. \n');
            fprintf('Please run gen_nCovE to create nCov3x3E.\n\n');
            fprintf('BTW, filename of nCov3x3E is %s\n',filename_nCov3x3E);
            fprintf('BTW, filename of nCov3x3w is %s\n',filename_nCov3x3w);
            return;
        end
    end

%% Prepare output file names
    filename_Eth_part = sprintf('_dpix_%d_dz_%d_r0_%d_esig_%.1f_Eth',...
        dpix, dz, r0, e_sig);
    for iEth=1:Nl
        filename_Eth_part = sprintf('%s_%.1f', filename_Eth_part, v_Eth(iEth));
    end
    filename_Eth_part_mat = sprintf('%s_keV.mat',filename_Eth_part);
    filename_Eth_part_flt = sprintf('%s_keV.flt',filename_Eth_part);
    
    filename_sino_pcd_mean_mat = sprintf('%s%s',filename_sino_pcd_mean, filename_Eth_part_mat);
    filename_sino_pcd_mean_flt = sprintf('%s%s',filename_sino_pcd_mean, filename_Eth_part_flt);
    
    filename_sino_pcd_noisy_mat = sprintf('%s%s',filename_sino_pcd_noisy, filename_Eth_part_mat);
    filename_sino_pcd_noisy_flt = sprintf('%s%s',filename_sino_pcd_noisy, filename_Eth_part_flt);

%%   Step 3. Calculate a 4-D spectrum sinogram from basis sinograms of multiple materials, St(E1,ch,row,view)
%    Step 4. Calculate a noise-free 4-D count sinogram, nR(Nl,ch,row,view)
%    Step 5. Generate a noisy 4-D count sinogram, nRm(Nl,ch,row,view)
% 
%   180 sec for 4 wins x 1854 ch x 7 rows x 2000 projs 
%
    fid_m = fopen(filename_sino_pcd_mean_flt, 'wb');
    fid_n = fopen(filename_sino_pcd_noisy_flt,'wb');
    m2_nSRFw = calc_SRF_from_Cov3x3( m3_nCov3x3w );
    m2_nSRFw = m2_nSRFw(:,1:length(v_S0E));

    for iv=1:Nview
        t1 = clock;
        if mod(iv,20)==0
            fprintf(' on view %d of %d: time = %d:%d:%.0f (hour:min:sec)\n', iv,Nview,t1(4),t1(5),t1(6));
        end
        m3_yt = zeros(Nl, Nch, Nrow);
        m3_yn = zeros(Nl, Nch, Nrow);
        
        for irow=1:Nrow
            
            %   Step 3. Calculate a 4-D spectrum sinogram from basis sinograms of multiple materials, St(E1,ch,row,view)
            m2_St = zeros(length(v_S0E),Nch);
            for ich=1:Nch
                m2_St(:,ich) = v_N0(iv).*v_S0E.*exp(-m2_mukE(:,1).*m4_sino_v(1,ich,irow,iv)-m2_mukE(:,2).*m4_sino_v(2,ich,irow,iv));
            end
            %   Step 4. Calculate a noise-free 4-D count sinogram, nR(Nl,ch,row,view)
                % gen pcd data (noise free)
            m2_yt = m2_nSRFw * m2_St;
            
            %   Step 5. Generate a noisy 4-D count sinogram, nRm(Nl,ch,row,view)
            switch iflag_noise
                case flag_noise_normal  % gen Gaussian data
                    m2_yn = normrnd(m2_yt, sqrt(m2_yt));        % ver 1.1
                case flag_noise_Poisson % gen Poisson data
                    m2_yn = random('Poisson', m2_yt);
            end
            m3_yt(:,:,irow) = m2_yt;
            m3_yn(:,:,irow) = m2_yn;
            
        end     % next irow
        
        fwrite( fid_m, m3_yt, 'float32' );
        fwrite( fid_n, m3_yn, 'float32' );
        
    end         % next iv
    
    fclose( fid_m );
    fclose( fid_n );
    
    % save the entire 4-D sinogram to .mat files
    fid_m = fopen(filename_sino_pcd_mean_flt, 'rb');
    m4_sino_pcd_mean=fread(fid_m, Nl*Nch*Nrow*Nview, 'float32');
    m4_sino_pcd_mean=reshape(m4_sino_pcd_mean, [Nl,Nch,Nrow,Nview]);
    save(filename_sino_pcd_mean_mat, 'm4_sino_pcd_mean','Nl','Nch','Nrow','Nview','-v7.3');

    fid_n = fopen(filename_sino_pcd_noisy_flt, 'rb');
    m4_sino_pcd_noisy=fread(fid_n, Nl*Nch*Nrow*Nview, 'float32');
    m4_sino_pcd_noisy=reshape(m4_sino_pcd_noisy, [Nl,Nch,Nrow,Nview]);
    save(filename_sino_pcd_noisy_mat, 'm4_sino_pcd_noisy','Nl','Nch','Nrow','Nview','-v7.3');

return;

